@@ -1,7 +1,6 @@
 """ hook specifications for pytest plugins, invoked from main.py and builtin plugins.  """
-from pluggy import HookspecMarker
 
-from _pytest.deprecated import PYTEST_LOGWARNING
+from _pytest._pluggy import HookspecMarker
 
 hookspec = HookspecMarker("pytest")
 
@@ -13,26 +12,22 @@ hookspec = HookspecMarker("pytest")
 @hookspec(historic=True)
 def pytest_addhooks(pluginmanager):
     """called at plugin registration time to allow adding new hooks via a call to
-    ``pluginmanager.add_hookspecs(module_or_class, prefix)``.
+    pluginmanager.add_hookspecs(module_or_class, prefix)."""
 
 
-    :param _pytest.config.PytestPluginManager pluginmanager: pytest plugin manager
-
-    .. note::
-        This hook is incompatible with ``hookwrapper=True``.
+@hookspec(historic=True)
+def pytest_namespace():
+    """
+    DEPRECATED: this hook causes direct monkeypatching on pytest, its use is strongly discouraged
+    return dict of name->object to be made globally available in
+    the pytest namespace.  This hook is called at plugin registration
+    time.
     """
 
 
 @hookspec(historic=True)
 def pytest_plugin_registered(plugin, manager):
-    """ a new pytest plugin got registered.
-
-    :param plugin: the plugin module or instance
-    :param _pytest.config.PytestPluginManager manager: pytest plugin manager
-
-    .. note::
-        This hook is incompatible with ``hookwrapper=True``.
-    """
+    """ a new pytest plugin got registered. """
 
 
 @hookspec(historic=True)
@@ -46,7 +41,7 @@ def pytest_addoption(parser):
         files situated at the tests root directory due to how pytest
         :ref:`discovers plugins during startup <pluginorder>`.
 
-    :arg _pytest.config.Parser parser: To add command line options, call
+    :arg parser: To add command line options, call
         :py:func:`parser.addoption(...) <_pytest.config.Parser.addoption>`.
         To add ini-file values call :py:func:`parser.addini(...)
         <_pytest.config.Parser.addini>`.
@@ -61,10 +56,8 @@ def pytest_addoption(parser):
       a value read from an ini-style file.
 
     The config object is passed around on many internal objects via the ``.config``
-    attribute or can be retrieved as the ``pytestconfig`` fixture.
-
-    .. note::
-        This hook is incompatible with ``hookwrapper=True``.
+    attribute or can be retrieved as the ``pytestconfig`` fixture or accessed
+    via (deprecated) ``pytest.config``.
     """
 
 
@@ -79,16 +72,14 @@ def pytest_configure(config):
     After that, the hook is called for other conftest files as they are
     imported.
 
-    .. note::
-        This hook is incompatible with ``hookwrapper=True``.
-
-    :arg _pytest.config.Config config: pytest config object
+    :arg config: pytest config object
+    :type config: _pytest.config.Config
     """
 
-
 # -------------------------------------------------------------------------
 # Bootstrapping hooks called for plugins registered early enough:
-# internal and 3rd party plugins.
+# internal and 3rd party plugins as well as directly
+# discoverable conftest.py local plugins.
 # -------------------------------------------------------------------------
 
 
@@ -96,28 +87,11 @@ def pytest_configure(config):
 def pytest_cmdline_parse(pluginmanager, args):
     """return initialized config object, parsing the specified args.
 
-    Stops at first non-None result, see :ref:`firstresult`
-
-    .. note::
-        This hook will not be called for ``conftest.py`` files, only for setuptools plugins.
-
-    :param _pytest.config.PytestPluginManager pluginmanager: pytest plugin manager
-    :param list[str] args: list of arguments passed on the command line
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
 def pytest_cmdline_preparse(config, args):
-    """(**Deprecated**) modify command line arguments before option parsing.
-
-    This hook is considered deprecated and will be removed in a future pytest version. Consider
-    using :func:`pytest_load_initial_conftests` instead.
-
-    .. note::
-        This hook will not be called for ``conftest.py`` files, only for setuptools plugins.
-
-    :param _pytest.config.Config config: pytest config object
-    :param list[str] args: list of arguments passed on the command line
-    """
+    """(deprecated) modify command line arguments before option parsing. """
 
 
 @hookspec(firstresult=True)
@@ -125,58 +99,32 @@ def pytest_cmdline_main(config):
     """ called for performing the main command line action. The default
     implementation will invoke the configure hooks and runtest_mainloop.
 
-    .. note::
-        This hook will not be called for ``conftest.py`` files, only for setuptools plugins.
-
-    Stops at first non-None result, see :ref:`firstresult`
-
-    :param _pytest.config.Config config: pytest config object
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
 def pytest_load_initial_conftests(early_config, parser, args):
     """ implements the loading of initial conftest files ahead
-    of command line option parsing.
-
-    .. note::
-        This hook will not be called for ``conftest.py`` files, only for setuptools plugins.
-
-    :param _pytest.config.Config early_config: pytest config object
-    :param list[str] args: list of arguments passed on the command line
-    :param _pytest.config.Parser parser: to add command line options
-    """
+    of command line option parsing. """
 
 
 # -------------------------------------------------------------------------
 # collection hooks
 # -------------------------------------------------------------------------
 
-
 @hookspec(firstresult=True)
 def pytest_collection(session):
-    """Perform the collection protocol for the given session.
+    """ perform the collection protocol for the given session.
 
-    Stops at first non-None result, see :ref:`firstresult`.
-
-    :param _pytest.main.Session session: the pytest session object
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
 def pytest_collection_modifyitems(session, config, items):
     """ called after collection has been performed, may filter or re-order
-    the items in-place.
-
-    :param _pytest.main.Session session: the pytest session object
-    :param _pytest.config.Config config: pytest config object
-    :param List[_pytest.nodes.Item] items: list of item objects
-    """
+    the items in-place."""
 
 
 def pytest_collection_finish(session):
-    """ called after collection has been performed and modified.
-
-    :param _pytest.main.Session session: the pytest session object
-    """
+    """ called after collection has been performed and modified. """
 
 
 @hookspec(firstresult=True)
@@ -186,9 +134,6 @@ def pytest_ignore_collect(path, config):
     more specific hooks.
 
     Stops at first non-None result, see :ref:`firstresult`
-
-    :param str path: the path to analyze
-    :param _pytest.config.Config config: pytest config object
     """
 
 
@@ -196,19 +141,12 @@ def pytest_ignore_collect(path, config):
 def pytest_collect_directory(path, parent):
     """ called before traversing a directory for collection files.
 
-    Stops at first non-None result, see :ref:`firstresult`
-
-    :param str path: the path to analyze
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
 def pytest_collect_file(path, parent):
     """ return collection Node or None for the given path. Any new node
-    needs to have the specified ``parent`` as a parent.
-
-    :param str path: the path to collect
-    """
-
+    needs to have the specified ``parent`` as a parent."""
 
 # logging hooks for collection
 
@@ -235,7 +173,6 @@ def pytest_make_collect_report(collector):
 
     Stops at first non-None result, see :ref:`firstresult` """
 
-
 # -------------------------------------------------------------------------
 # Python test function related hooks
 # -------------------------------------------------------------------------
@@ -275,13 +212,7 @@ def pytest_make_parametrize_id(config, val, argname):
     by @pytest.mark.parametrize calls. Return None if the hook doesn't know about ``val``.
     The parameter name is available as ``argname``, if required.
 
-    Stops at first non-None result, see :ref:`firstresult`
-
-    :param _pytest.config.Config config: pytest config object
-    :param val: the parametrized value
-    :param str argname: the automatic parameter name produced by pytest
-    """
-
+    Stops at first non-None result, see :ref:`firstresult` """
 
 # -------------------------------------------------------------------------
 # generic runtest related hooks
@@ -293,14 +224,11 @@ def pytest_runtestloop(session):
     """ called for performing the main runtest loop
     (after collection finished).
 
-    Stops at first non-None result, see :ref:`firstresult`
-
-    :param _pytest.main.Session session: the pytest session object
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
 def pytest_itemstart(item, node):
-    """(**Deprecated**) use pytest_runtest_logstart. """
+    """ (deprecated, use pytest_runtest_logstart). """
 
 
 @hookspec(firstresult=True)
@@ -322,25 +250,7 @@ def pytest_runtest_protocol(item, nextitem):
 
 
 def pytest_runtest_logstart(nodeid, location):
-    """ signal the start of running a single test item.
-
-    This hook will be called **before** :func:`pytest_runtest_setup`, :func:`pytest_runtest_call` and
-    :func:`pytest_runtest_teardown` hooks.
-
-    :param str nodeid: full id of the item
-    :param location: a triple of ``(filename, linenum, testname)``
-    """
-
-
-def pytest_runtest_logfinish(nodeid, location):
-    """ signal the complete finish of running a single test item.
-
-    This hook will be called **after** :func:`pytest_runtest_setup`, :func:`pytest_runtest_call` and
-    :func:`pytest_runtest_teardown` hooks.
-
-    :param str nodeid: full id of the item
-    :param location: a triple of ``(filename, linenum, testname)``
-    """
+    """ signal the start of running a single test item. """
 
 
 def pytest_runtest_setup(item):
@@ -374,7 +284,6 @@ def pytest_runtest_logreport(report):
     """ process a test setup/call/teardown report relating to
     the respective phase of executing a test. """
 
-
 # -------------------------------------------------------------------------
 # Fixture related hooks
 # -------------------------------------------------------------------------
@@ -384,56 +293,35 @@ def pytest_runtest_logreport(report):
 def pytest_fixture_setup(fixturedef, request):
     """ performs fixture setup execution.
 
-    :return: The return value of the call to the fixture function
-
-    Stops at first non-None result, see :ref:`firstresult`
-
-    .. note::
-        If the fixture function returns None, other implementations of
-        this hook function will continue to be called, according to the
-        behavior of the :ref:`firstresult` option.
-    """
+    Stops at first non-None result, see :ref:`firstresult` """
 
 
-def pytest_fixture_post_finalizer(fixturedef, request):
+def pytest_fixture_post_finalizer(fixturedef):
     """ called after fixture teardown, but before the cache is cleared so
     the fixture result cache ``fixturedef.cached_result`` can
     still be accessed."""
 
-
 # -------------------------------------------------------------------------
 # test session related hooks
 # -------------------------------------------------------------------------
 
 
 def pytest_sessionstart(session):
-    """ called after the ``Session`` object has been created and before performing collection
-    and entering the run test loop.
-
-    :param _pytest.main.Session session: the pytest session object
-    """
+    """ before session.main() is called. """
 
 
 def pytest_sessionfinish(session, exitstatus):
-    """ called after whole test run finished, right before returning the exit status to the system.
-
-    :param _pytest.main.Session session: the pytest session object
-    :param int exitstatus: the status which pytest will return to the system
-    """
+    """ whole test run finishes. """
 
 
 def pytest_unconfigure(config):
-    """ called before test process is exited.
-
-    :param _pytest.config.Config config: pytest config object
-    """
+    """ called before test process is exited.  """
 
 
 # -------------------------------------------------------------------------
 # hooks for customizing the assert methods
 # -------------------------------------------------------------------------
 
-
 def pytest_assertrepr_compare(config, op, left, right):
     """return explanation for comparisons in failing assert expressions.
 
@@ -441,11 +329,8 @@ def pytest_assertrepr_compare(config, op, left, right):
     of strings.  The strings will be joined by newlines but any newlines
     *in* a string will be escaped.  Note that all but the first line will
     be indented slightly, the intention is for the first line to be a summary.
-
-    :param _pytest.config.Config config: pytest config object
     """
 
-
 # -------------------------------------------------------------------------
 # hooks for influencing reporting (invoked from _pytest_terminal)
 # -------------------------------------------------------------------------
@@ -454,7 +339,7 @@ def pytest_assertrepr_compare(config, op, left, right):
 def pytest_report_header(config, startdir):
     """ return a string or list of strings to be displayed as header info for terminal reporting.
 
-    :param _pytest.config.Config config: pytest config object
+    :param config: the pytest config object.
     :param startdir: py.path object with the starting dir
 
     .. note::
@@ -473,77 +358,28 @@ def pytest_report_collectionfinish(config, startdir, items):
 
     This strings will be displayed after the standard "collected X items" message.
 
-    :param _pytest.config.Config config: pytest config object
+    :param config: the pytest config object.
     :param startdir: py.path object with the starting dir
     :param items: list of pytest items that are going to be executed; this list should not be modified.
     """
 
 
 @hookspec(firstresult=True)
-def pytest_report_teststatus(report, config):
+def pytest_report_teststatus(report):
     """ return result-category, shortletter and verbose word for reporting.
 
-    :param _pytest.config.Config config: pytest config object
-
     Stops at first non-None result, see :ref:`firstresult` """
 
 
-def pytest_terminal_summary(terminalreporter, exitstatus, config):
-    """Add a section to terminal summary reporting.
-
-    :param _pytest.terminal.TerminalReporter terminalreporter: the internal terminal reporter object
-    :param int exitstatus: the exit status that will be reported back to the OS
-    :param _pytest.config.Config config: pytest config object
-
-    .. versionadded:: 4.2
-        The ``config`` parameter.
-    """
-
-
-@hookspec(historic=True, warn_on_impl=PYTEST_LOGWARNING)
-def pytest_logwarning(message, code, nodeid, fslocation):
-    """
-    .. deprecated:: 3.8
-
-        This hook is will stop working in a future release.
-
-        pytest no longer triggers this hook, but the
-        terminal writer still implements it to display warnings issued by
-        :meth:`_pytest.config.Config.warn` and :meth:`_pytest.nodes.Node.warn`. Calling those functions will be
-        an error in future releases.
-
-    process a warning specified by a message, a code string,
-    a nodeid and fslocation (both of which may be None
-    if the warning is not tied to a particular node/location).
-
-    .. note::
-        This hook is incompatible with ``hookwrapper=True``.
-    """
+def pytest_terminal_summary(terminalreporter, exitstatus):
+    """ add additional section in terminal summary reporting.  """
 
 
 @hookspec(historic=True)
-def pytest_warning_captured(warning_message, when, item):
-    """
-    Process a warning captured by the internal pytest warnings plugin.
-
-    :param warnings.WarningMessage warning_message:
-        The captured warning. This is the same object produced by :py:func:`warnings.catch_warnings`, and contains
-        the same attributes as the parameters of :py:func:`warnings.showwarning`.
-
-    :param str when:
-        Indicates when the warning was captured. Possible values:
-
-        * ``"config"``: during pytest configuration/initialization stage.
-        * ``"collect"``: during test collection.
-        * ``"runtest"``: during test execution.
-
-    :param pytest.Item|None item:
-        **DEPRECATED**: This parameter is incompatible with ``pytest-xdist``, and will always receive ``None``
-        in a future release.
-
-        The item being executed if ``when`` is ``"runtest"``, otherwise ``None``.
-    """
-
+def pytest_logwarning(message, code, nodeid, fslocation):
+    """ process a warning specified by a message, a code string,
+    a nodeid and fslocation (both of which may be None
+    if the warning is not tied to a partilar node/location)."""
 
 # -------------------------------------------------------------------------
 # doctest hooks
@@ -556,7 +392,6 @@ def pytest_doctest_prepare_content(content):
 
     Stops at first non-None result, see :ref:`firstresult` """
 
-
 # -------------------------------------------------------------------------
 # error handling and internal debugging hooks
 # -------------------------------------------------------------------------
@@ -579,21 +414,10 @@ def pytest_exception_interact(node, call, report):
     """
 
 
-def pytest_enter_pdb(config, pdb):
+def pytest_enter_pdb(config):
     """ called upon pdb.set_trace(), can be used by plugins to take special
     action just before the python debugger enters in interactive mode.
 
-    :param _pytest.config.Config config: pytest config object
-    :param pdb.Pdb pdb: Pdb instance
-    """
-
-
-def pytest_leave_pdb(config, pdb):
-    """ called when leaving pdb (e.g. with continue after pdb.set_trace()).
-
-    Can be used by plugins to take special action just after the python
-    debugger leaves interactive mode.
-
-    :param _pytest.config.Config config: pytest config object
-    :param pdb.Pdb pdb: Pdb instance
+    :arg config: pytest config object
+    :type config: _pytest.config.Config
     """
