@@ -1,22 +1,14 @@
-# -*- coding: utf-8 -*-
 """ monkeypatching and mocking functionality.  """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
 
 import os
-import re
 import sys
-import warnings
-from contextlib import contextmanager
+import re
 
-import six
-
-import pytest
+from py.builtin import _basestring
 from _pytest.fixtures import fixture
-from _pytest.pathlib import Path
 
-RE_IMPORT_ERROR_NAME = re.compile(r"^No module named (.*)$")
+RE_IMPORT_ERROR_NAME = re.compile("^No module named (.*)$")
 
 
 @fixture
@@ -29,7 +21,7 @@ def monkeypatch():
         monkeypatch.setitem(mapping, name, value)
         monkeypatch.delitem(obj, name, raising=True)
         monkeypatch.setenv(name, value, prepend=False)
-        monkeypatch.delenv(name, raising=True)
+        monkeypatch.delenv(name, value, raising=True)
         monkeypatch.syspath_prepend(path)
         monkeypatch.chdir(path)
 
@@ -45,12 +37,12 @@ def monkeypatch():
 
 def resolve(name):
     # simplified from zope.dottedname
-    parts = name.split(".")
+    parts = name.split('.')
 
     used = parts.pop(0)
     found = __import__(used)
     for part in parts:
-        used += "." + part
+        used += '.' + part
         try:
             found = getattr(found, part)
         except AttributeError:
@@ -67,7 +59,9 @@ def resolve(name):
             if expected == used:
                 raise
             else:
-                raise ImportError("import error in %s: %s" % (used, ex))
+                raise ImportError(
+                    'import error in %s: %s' % (used, ex)
+                )
         found = annotated_getattr(found, part, used)
     return found
 
@@ -77,22 +71,25 @@ def annotated_getattr(obj, name, ann):
         obj = getattr(obj, name)
     except AttributeError:
         raise AttributeError(
-            "%r object at %s has no attribute %r" % (type(obj).__name__, ann, name)
+            '%r object at %s has no attribute %r' % (
+                type(obj).__name__, ann, name
+            )
         )
     return obj
 
 
 def derive_importpath(import_path, raising):
-    if not isinstance(import_path, six.string_types) or "." not in import_path:
-        raise TypeError("must be absolute import path string, not %r" % (import_path,))
-    module, attr = import_path.rsplit(".", 1)
+    if not isinstance(import_path, _basestring) or "." not in import_path:
+        raise TypeError("must be absolute import path string, not %r" %
+                        (import_path,))
+    module, attr = import_path.rsplit('.', 1)
     target = resolve(module)
     if raising:
         annotated_getattr(target, attr, ann=module)
     return attr, target
 
 
-class Notset(object):
+class Notset:
     def __repr__(self):
         return "<notset>"
 
@@ -100,7 +97,7 @@ class Notset(object):
 notset = Notset()
 
 
-class MonkeyPatch(object):
+class MonkeyPatch:
     """ Object returned by the ``monkeypatch`` fixture keeping a record of setattr/item/env/syspath changes.
     """
 
@@ -110,29 +107,6 @@ class MonkeyPatch(object):
         self._cwd = None
         self._savesyspath = None
 
-    @contextmanager
-    def context(self):
-        """
-        Context manager that returns a new :class:`MonkeyPatch` object which
-        undoes any patching done inside the ``with`` block upon exit:
-
-        .. code-block:: python
-
-            import functools
-            def test_partial(monkeypatch):
-                with monkeypatch.context() as m:
-                    m.setattr(functools, "partial", 3)
-
-        Useful in situations where it is desired to undo some patches before the test ends,
-        such as mocking ``stdlib`` functions that might break pytest itself if mocked (for examples
-        of this see `#3290 <https://github.com/pytest-dev/pytest/issues/3290>`_.
-        """
-        m = MonkeyPatch()
-        try:
-            yield m
-        finally:
-            m.undo()
-
     def setattr(self, target, name, value=notset, raising=True):
         """ Set attribute value on target, memorizing the old value.
         By default raise AttributeError if the attribute did not exist.
@@ -140,7 +114,7 @@ class MonkeyPatch(object):
         For convenience you can specify a string as ``target`` which
         will be interpreted as a dotted import path, with the last part
         being the attribute name.  Example:
-        ``monkeypatch.setattr("os.getcwd", lambda: "/")``
+        ``monkeypatch.setattr("os.getcwd", lambda x: "/")``
         would set the ``getcwd`` function of the ``os`` module.
 
         The ``raising`` value determines if the setattr should fail
@@ -151,12 +125,10 @@ class MonkeyPatch(object):
         import inspect
 
         if value is notset:
-            if not isinstance(target, six.string_types):
-                raise TypeError(
-                    "use setattr(target, name, value) or "
-                    "setattr(target, value) with target being a dotted "
-                    "import string"
-                )
+            if not isinstance(target, _basestring):
+                raise TypeError("use setattr(target, name, value) or "
+                                "setattr(target, value) with target being a dotted "
+                                "import string")
             value = name
             name, target = derive_importpath(target, raising)
 
@@ -182,26 +154,18 @@ class MonkeyPatch(object):
         attribute is missing.
         """
         __tracebackhide__ = True
-        import inspect
-
         if name is notset:
-            if not isinstance(target, six.string_types):
-                raise TypeError(
-                    "use delattr(target, name) or "
-                    "delattr(target) with target being a dotted "
-                    "import string"
-                )
+            if not isinstance(target, _basestring):
+                raise TypeError("use delattr(target, name) or "
+                                "delattr(target) with target being a dotted "
+                                "import string")
             name, target = derive_importpath(target, raising)
 
         if not hasattr(target, name):
             if raising:
                 raise AttributeError(name)
         else:
-            oldval = getattr(target, name, notset)
-            # Avoid class descriptors like staticmethod/classmethod.
-            if inspect.isclass(target):
-                oldval = target.__dict__.get(name, notset)
-            self._setattr.append((target, name, oldval))
+            self._setattr.append((target, name, getattr(target, name, notset)))
             delattr(target, name)
 
     def setitem(self, dic, name, value):
@@ -222,68 +186,30 @@ class MonkeyPatch(object):
             self._setitem.append((dic, name, dic.get(name, notset)))
             del dic[name]
 
-    def _warn_if_env_name_is_not_str(self, name):
-        """On Python 2, warn if the given environment variable name is not a native str (#4056)"""
-        if six.PY2 and not isinstance(name, str):
-            warnings.warn(
-                pytest.PytestWarning(
-                    "Environment variable name {!r} should be str".format(name)
-                )
-            )
-
     def setenv(self, name, value, prepend=None):
         """ Set environment variable ``name`` to ``value``.  If ``prepend``
         is a character, read the current environment variable value
         and prepend the ``value`` adjoined with the ``prepend`` character."""
-        if not isinstance(value, str):
-            warnings.warn(
-                pytest.PytestWarning(
-                    "Value of environment variable {name} type should be str, but got "
-                    "{value!r} (type: {type}); converted to str implicitly".format(
-                        name=name, value=value, type=type(value).__name__
-                    )
-                ),
-                stacklevel=2,
-            )
-            value = str(value)
+        value = str(value)
         if prepend and name in os.environ:
             value = value + prepend + os.environ[name]
-        self._warn_if_env_name_is_not_str(name)
         self.setitem(os.environ, name, value)
 
     def delenv(self, name, raising=True):
-        """ Delete ``name`` from the environment. Raise KeyError if it does
-        not exist.
+        """ Delete ``name`` from the environment. Raise KeyError it does not
+        exist.
 
         If ``raising`` is set to False, no exception will be raised if the
         environment variable is missing.
         """
-        self._warn_if_env_name_is_not_str(name)
         self.delitem(os.environ, name, raising=raising)
 
     def syspath_prepend(self, path):
         """ Prepend ``path`` to ``sys.path`` list of import locations. """
-        from pkg_resources import fixup_namespace_packages
-
         if self._savesyspath is None:
             self._savesyspath = sys.path[:]
         sys.path.insert(0, str(path))
 
-        # https://github.com/pypa/setuptools/blob/d8b901bc/docs/pkg_resources.txt#L162-L171
-        fixup_namespace_packages(str(path))
-
-        # A call to syspathinsert() usually means that the caller wants to
-        # import some dynamically created files, thus with python3 we
-        # invalidate its import caches.
-        # This is especially important when any namespace package is in used,
-        # since then the mtime based FileFinder cache (that gets created in
-        # this case already) gets not invalidated when writing the new files
-        # quickly afterwards.
-        if sys.version_info >= (3, 3):
-            from importlib import invalidate_caches
-
-            invalidate_caches()
-
     def chdir(self, path):
         """ Change the current working directory to the specified path.
         Path can be a string or a py.path.local object.
@@ -292,9 +218,6 @@ class MonkeyPatch(object):
             self._cwd = os.getcwd()
         if hasattr(path, "chdir"):
             path.chdir()
-        elif isinstance(path, Path):
-            # modern python uses the fspath protocol here LEGACY
-            os.chdir(str(path))
         else:
             os.chdir(path)
 
