@@ -1,27 +1,11 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-
 import math
-import pprint
 import sys
-import warnings
-from decimal import Decimal
-from numbers import Number
 
-from more_itertools.more import always_iterable
-from six.moves import filterfalse
-from six.moves import zip
+import py
 
-import _pytest._code
-from _pytest import deprecated
-from _pytest.compat import isclass
-from _pytest.compat import Iterable
-from _pytest.compat import Mapping
-from _pytest.compat import Sized
-from _pytest.compat import STRING_TYPES
+from _pytest.compat import isclass, izip
 from _pytest.outcomes import fail
-
-BASE_TYPE = (type, STRING_TYPES)
+import _pytest._code
 
 
 def _cmp_raises_type_error(self, other):
@@ -33,18 +17,7 @@ def _cmp_raises_type_error(self, other):
     other operators at all.
     """
     __tracebackhide__ = True
-    raise TypeError(
-        "Comparison operators other than == and != not supported by approx objects"
-    )
-
-
-def _non_numeric_type_error(value, at):
-    at_str = " at {}".format(at) if at else ""
-    return TypeError(
-        "cannot make approximate comparisons to non-numeric values: {!r} {}".format(
-            value, at_str
-        )
-    )
+    raise TypeError('Comparison operators other than == and != not supported by approx objects')
 
 
 # builtin pytest.approx helper
@@ -56,25 +29,19 @@ class ApproxBase(object):
     or sequences of numbers.
     """
 
-    # Tell numpy to use our `__eq__` operator instead of its.
-    __array_ufunc__ = None
-    __array_priority__ = 100
-
     def __init__(self, expected, rel=None, abs=None, nan_ok=False):
-        __tracebackhide__ = True
         self.expected = expected
         self.abs = abs
         self.rel = rel
         self.nan_ok = nan_ok
-        self._check_type()
 
     def __repr__(self):
         raise NotImplementedError
 
     def __eq__(self, actual):
         return all(
-            a == self._approx_scalar(x) for a, x in self._yield_comparisons(actual)
-        )
+            a == self._approx_scalar(x)
+            for a, x in self._yield_comparisons(actual))
 
     __hash__ = None
 
@@ -94,33 +61,20 @@ class ApproxBase(object):
         """
         raise NotImplementedError
 
-    def _check_type(self):
-        """
-        Raise a TypeError if the expected value is not a valid type.
-        """
-        # This is only a concern if the expected value is a sequence.  In every
-        # other case, the approx() function ensures that the expected value has
-        # a numeric type.  For this reason, the default is to do nothing.  The
-        # classes that deal with sequences should reimplement this method to
-        # raise if there are any non-numeric elements in the sequence.
-        pass
-
-
-def _recursive_list_map(f, x):
-    if isinstance(x, list):
-        return list(_recursive_list_map(f, xi) for xi in x)
-    else:
-        return f(x)
-
 
 class ApproxNumpy(ApproxBase):
     """
-    Perform approximate comparisons where the expected value is numpy array.
+    Perform approximate comparisons for numpy arrays.
     """
 
+    # Tell numpy to use our `__eq__` operator instead of its.
+    __array_priority__ = 100
+
     def __repr__(self):
-        list_scalars = _recursive_list_map(self._approx_scalar, self.expected.tolist())
-        return "approx({!r})".format(list_scalars)
+        # It might be nice to rewrite this function to account for the
+        # shape of the array...
+        return "approx({0!r})".format(list(
+            self._approx_scalar(x) for x in self.expected))
 
     if sys.version_info[0] == 2:
         __cmp__ = _cmp_raises_type_error
@@ -128,15 +82,12 @@ class ApproxNumpy(ApproxBase):
     def __eq__(self, actual):
         import numpy as np
 
-        # self.expected is supposed to always be an array here
+        try:
+            actual = np.asarray(actual)
+        except:  # noqa
+            raise TypeError("cannot compare '{0}' to numpy.ndarray".format(actual))
 
-        if not np.isscalar(actual):
-            try:
-                actual = np.asarray(actual)
-            except:  # noqa
-                raise TypeError("cannot compare '{}' to numpy.ndarray".format(actual))
-
-        if not np.isscalar(actual) and actual.shape != self.expected.shape:
+        if actual.shape != self.expected.shape:
             return False
 
         return ApproxBase.__eq__(self, actual)
@@ -144,28 +95,23 @@ class ApproxNumpy(ApproxBase):
     def _yield_comparisons(self, actual):
         import numpy as np
 
-        # `actual` can either be a numpy array or a scalar, it is treated in
-        # `__eq__` before being passed to `ApproxBase.__eq__`, which is the
-        # only method that calls this one.
-
-        if np.isscalar(actual):
-            for i in np.ndindex(self.expected.shape):
-                yield actual, self.expected[i].item()
-        else:
-            for i in np.ndindex(self.expected.shape):
-                yield actual[i].item(), self.expected[i].item()
+        # We can be sure that `actual` is a numpy array, because it's
+        # casted in `__eq__` before being passed to `ApproxBase.__eq__`,
+        # which is the only method that calls this one.
+        for i in np.ndindex(self.expected.shape):
+            yield actual[i], self.expected[i]
 
 
 class ApproxMapping(ApproxBase):
     """
-    Perform approximate comparisons where the expected value is a mapping with
-    numeric values (the keys can be anything).
+    Perform approximate comparisons for mappings where the values are numbers
+    (the keys can be anything).
     """
 
     def __repr__(self):
-        return "approx({!r})".format(
-            {k: self._approx_scalar(v) for k, v in self.expected.items()}
-        )
+        return "approx({0!r})".format(dict(
+            (k, self._approx_scalar(v))
+            for k, v in self.expected.items()))
 
     def __eq__(self, actual):
         if set(actual.keys()) != set(self.expected.keys()):
@@ -177,29 +123,21 @@ class ApproxMapping(ApproxBase):
         for k in self.expected.keys():
             yield actual[k], self.expected[k]
 
-    def _check_type(self):
-        __tracebackhide__ = True
-        for key, value in self.expected.items():
-            if isinstance(value, type(self.expected)):
-                msg = "pytest.approx() does not support nested dictionaries: key={!r} value={!r}\n  full mapping={}"
-                raise TypeError(msg.format(key, value, pprint.pformat(self.expected)))
-            elif not isinstance(value, Number):
-                raise _non_numeric_type_error(self.expected, at="key={!r}".format(key))
 
+class ApproxSequence(ApproxBase):
+    """
+    Perform approximate comparisons for sequences of numbers.
+    """
 
-class ApproxSequencelike(ApproxBase):
-    """
-    Perform approximate comparisons where the expected value is a sequence of
-    numbers.
-    """
+    # Tell numpy to use our `__eq__` operator instead of its.
+    __array_priority__ = 100
 
     def __repr__(self):
         seq_type = type(self.expected)
         if seq_type not in (tuple, list, set):
             seq_type = list
-        return "approx({!r})".format(
-            seq_type(self._approx_scalar(x) for x in self.expected)
-        )
+        return "approx({0!r})".format(seq_type(
+            self._approx_scalar(x) for x in self.expected))
 
     def __eq__(self, actual):
         if len(actual) != len(self.expected):
@@ -207,28 +145,14 @@ class ApproxSequencelike(ApproxBase):
         return ApproxBase.__eq__(self, actual)
 
     def _yield_comparisons(self, actual):
-        return zip(actual, self.expected)
-
-    def _check_type(self):
-        __tracebackhide__ = True
-        for index, x in enumerate(self.expected):
-            if isinstance(x, type(self.expected)):
-                msg = "pytest.approx() does not support nested data structures: {!r} at index {}\n  full sequence: {}"
-                raise TypeError(msg.format(x, index, pprint.pformat(self.expected)))
-            elif not isinstance(x, Number):
-                raise _non_numeric_type_error(
-                    self.expected, at="index {}".format(index)
-                )
+        return izip(actual, self.expected)
 
 
 class ApproxScalar(ApproxBase):
     """
-    Perform approximate comparisons where the expected value is a single number.
+    Perform approximate comparisons for single numbers only.
     """
 
-    DEFAULT_ABSOLUTE_TOLERANCE = 1e-12
-    DEFAULT_RELATIVE_TOLERANCE = 1e-6
-
     def __repr__(self):
         """
         Return a string communicating both the expected value and the tolerance
@@ -247,24 +171,20 @@ class ApproxScalar(ApproxBase):
         # If a sensible tolerance can't be calculated, self.tolerance will
         # raise a ValueError.  In this case, display '???'.
         try:
-            vetted_tolerance = "{:.1e}".format(self.tolerance)
+            vetted_tolerance = '{:.1e}'.format(self.tolerance)
         except ValueError:
-            vetted_tolerance = "???"
+            vetted_tolerance = '???'
 
         if sys.version_info[0] == 2:
-            return "{} +- {}".format(self.expected, vetted_tolerance)
+            return '{0} +- {1}'.format(self.expected, vetted_tolerance)
         else:
-            return u"{} \u00b1 {}".format(self.expected, vetted_tolerance)
+            return u'{0} \u00b1 {1}'.format(self.expected, vetted_tolerance)
 
     def __eq__(self, actual):
         """
         Return true if the given value is equal to the expected value within
         the pre-specified tolerance.
         """
-        if _is_numpy_array(actual):
-            # Call ``__eq__()`` manually to prevent infinite-recursion with
-            # numpy<1.13.  See #3748.
-            return all(self.__eq__(a) for a in actual.flat)
 
         # Short-circuit exact equality.
         if actual == self.expected:
@@ -297,18 +217,15 @@ class ApproxScalar(ApproxBase):
         absolute tolerance or a relative tolerance, depending on what the user
         specified or which would be larger.
         """
-
         def set_default(x, default):
             return x if x is not None else default
 
         # Figure out what the absolute tolerance should be.  ``self.abs`` is
         # either None or a value specified by the user.
-        absolute_tolerance = set_default(self.abs, self.DEFAULT_ABSOLUTE_TOLERANCE)
+        absolute_tolerance = set_default(self.abs, 1e-12)
 
         if absolute_tolerance < 0:
-            raise ValueError(
-                "absolute tolerance can't be negative: {}".format(absolute_tolerance)
-            )
+            raise ValueError("absolute tolerance can't be negative: {}".format(absolute_tolerance))
         if math.isnan(absolute_tolerance):
             raise ValueError("absolute tolerance can't be NaN.")
 
@@ -323,14 +240,10 @@ class ApproxScalar(ApproxBase):
         # we've made sure the user didn't ask for an absolute tolerance only,
         # because we don't want to raise errors about the relative tolerance if
         # we aren't even going to use it.
-        relative_tolerance = set_default(
-            self.rel, self.DEFAULT_RELATIVE_TOLERANCE
-        ) * abs(self.expected)
+        relative_tolerance = set_default(self.rel, 1e-6) * abs(self.expected)
 
         if relative_tolerance < 0:
-            raise ValueError(
-                "relative tolerance can't be negative: {}".format(absolute_tolerance)
-            )
+            raise ValueError("relative tolerance can't be negative: {}".format(absolute_tolerance))
         if math.isnan(relative_tolerance):
             raise ValueError("relative tolerance can't be NaN.")
 
@@ -338,15 +251,6 @@ class ApproxScalar(ApproxBase):
         return max(relative_tolerance, absolute_tolerance)
 
 
-class ApproxDecimal(ApproxScalar):
-    """
-    Perform approximate comparisons where the expected value is a decimal.
-    """
-
-    DEFAULT_ABSOLUTE_TOLERANCE = Decimal("1e-12")
-    DEFAULT_RELATIVE_TOLERANCE = Decimal("1e-6")
-
-
 def approx(expected, rel=None, abs=None, nan_ok=False):
     """
     Assert that two numbers (or two sets of numbers) are equal to each other
@@ -393,18 +297,12 @@ def approx(expected, rel=None, abs=None, nan_ok=False):
         >>> {'a': 0.1 + 0.2, 'b': 0.2 + 0.4} == approx({'a': 0.3, 'b': 0.6})
         True
 
-    ``numpy`` arrays::
+    And ``numpy`` arrays::
 
         >>> import numpy as np                                                          # doctest: +SKIP
         >>> np.array([0.1, 0.2]) + np.array([0.2, 0.4]) == approx(np.array([0.3, 0.6])) # doctest: +SKIP
         True
 
-    And for a ``numpy`` array against a scalar::
-
-        >>> import numpy as np                                         # doctest: +SKIP
-        >>> np.array([0.1, 0.2]) + np.array([0.2, 0.1]) == approx(0.3) # doctest: +SKIP
-        True
-
     By default, ``approx`` considers numbers within a relative tolerance of
     ``1e-6`` (i.e. one part in a million) of its expected value to be equal.
     This treatment would lead to surprising results if the expected value was
@@ -500,39 +398,31 @@ def approx(expected, rel=None, abs=None, nan_ok=False):
        __ https://docs.python.org/3/reference/datamodel.html#object.__ge__
     """
 
+    from collections import Mapping, Sequence
+    from _pytest.compat import STRING_TYPES as String
+
     # Delegate the comparison to a class that knows how to deal with the type
     # of the expected value (e.g. int, float, list, dict, numpy.array, etc).
     #
-    # The primary responsibility of these classes is to implement ``__eq__()``
-    # and ``__repr__()``.  The former is used to actually check if some
-    # "actual" value is equivalent to the given expected value within the
-    # allowed tolerance.  The latter is used to show the user the expected
-    # value and tolerance, in the case that a test failed.
+    # This architecture is really driven by the need to support numpy arrays.
+    # The only way to override `==` for arrays without requiring that approx be
+    # the left operand is to inherit the approx object from `numpy.ndarray`.
+    # But that can't be a general solution, because it requires (1) numpy to be
+    # installed and (2) the expected value to be a numpy array.  So the general
+    # solution is to delegate each type of expected value to a different class.
     #
-    # The actual logic for making approximate comparisons can be found in
-    # ApproxScalar, which is used to compare individual numbers.  All of the
-    # other Approx classes eventually delegate to this class.  The ApproxBase
-    # class provides some convenient methods and overloads, but isn't really
-    # essential.
+    # This has the advantage that it made it easy to support mapping types
+    # (i.e. dict).  The old code accepted mapping types, but would only compare
+    # their keys, which is probably not what most people would expect.
 
-    __tracebackhide__ = True
-
-    if isinstance(expected, Decimal):
-        cls = ApproxDecimal
-    elif isinstance(expected, Number):
-        cls = ApproxScalar
+    if _is_numpy_array(expected):
+        cls = ApproxNumpy
     elif isinstance(expected, Mapping):
         cls = ApproxMapping
-    elif _is_numpy_array(expected):
-        cls = ApproxNumpy
-    elif (
-        isinstance(expected, Iterable)
-        and isinstance(expected, Sized)
-        and not isinstance(expected, STRING_TYPES)
-    ):
-        cls = ApproxSequencelike
+    elif isinstance(expected, Sequence) and not isinstance(expected, String):
+        cls = ApproxSequence
     else:
-        raise _non_numeric_type_error(expected, at=None)
+        cls = ApproxScalar
 
     return cls(expected, rel, abs, nan_ok)
 
@@ -542,68 +432,44 @@ def _is_numpy_array(obj):
     Return true if the given object is a numpy array.  Make a special effort to
     avoid importing numpy unless it's really necessary.
     """
-    import sys
+    import inspect
+
+    for cls in inspect.getmro(type(obj)):
+        if cls.__module__ == 'numpy':
+            try:
+                import numpy as np
+                return isinstance(obj, np.ndarray)
+            except ImportError:
+                pass
 
-    np = sys.modules.get("numpy")
-    if np is not None:
-        return isinstance(obj, np.ndarray)
     return False
 
 
 # builtin pytest.raises helper
 
-
 def raises(expected_exception, *args, **kwargs):
-    r"""
+    """
     Assert that a code block/function call raises ``expected_exception``
-    or raise a failure exception otherwise.
+    and raise a failure exception otherwise.
 
-    :kwparam match: if specified, a string containing a regular expression,
-        or a regular expression object, that is tested against the string
-        representation of the exception using ``re.search``. To match a literal
-        string that may contain `special characters`__, the pattern can
-        first be escaped with ``re.escape``.
+    This helper produces a ``ExceptionInfo()`` object (see below).
 
-    __ https://docs.python.org/3/library/re.html#regular-expression-syntax
-
-    :kwparam message: **(deprecated since 4.1)** if specified, provides a custom failure message
-        if the exception is not raised. See :ref:`the deprecation docs <raises message deprecated>` for a workaround.
-
-    .. currentmodule:: _pytest._code
-
-    Use ``pytest.raises`` as a context manager, which will capture the exception of the given
-    type::
+    If using Python 2.5 or above, you may use this function as a
+    context manager::
 
         >>> with raises(ZeroDivisionError):
         ...    1/0
 
-    If the code block does not raise the expected exception (``ZeroDivisionError`` in the example
-    above), or no exception at all, the check will fail instead.
+    .. versionchanged:: 2.10
 
-    You can also use the keyword argument ``match`` to assert that the
-    exception matches a text or regex::
+    In the context manager form you may use the keyword argument
+    ``message`` to specify a custom failure message::
 
-        >>> with raises(ValueError, match='must be 0 or None'):
-        ...     raise ValueError("value must be 0 or None")
-
-        >>> with raises(ValueError, match=r'must be \d+$'):
-        ...     raise ValueError("value must be 42")
-
-    The context manager produces an :class:`ExceptionInfo` object which can be used to inspect the
-    details of the captured exception::
-
-        >>> with raises(ValueError) as exc_info:
-        ...     raise ValueError("value must be 42")
-        >>> assert exc_info.type is ValueError
-        >>> assert exc_info.value.args[0] == "value must be 42"
-
-    .. deprecated:: 4.1
-
-        In the context manager form you may use the keyword argument
-        ``message`` to specify a custom failure message that will be displayed
-        in case the ``pytest.raises`` check fails. This has been deprecated as it
-        is considered error prone as users often mean to use ``match`` instead.
-        See :ref:`the deprecation docs <raises message deprecated>` for a workaround.
+        >>> with raises(ZeroDivisionError, message="Expecting ZeroDivisionError"):
+        ...    pass
+        Traceback (most recent call last):
+          ...
+        Failed: Expecting ZeroDivisionError
 
     .. note::
 
@@ -617,7 +483,7 @@ def raises(expected_exception, *args, **kwargs):
            >>> with raises(ValueError) as exc_info:
            ...     if value > 10:
            ...         raise ValueError("value must be <= 10")
-           ...     assert exc_info.type is ValueError  # this will not execute
+           ...     assert exc_info.type == ValueError  # this will not execute
 
        Instead, the following approach must be taken (note the difference in
        scope)::
@@ -626,17 +492,22 @@ def raises(expected_exception, *args, **kwargs):
            ...     if value > 10:
            ...         raise ValueError("value must be <= 10")
            ...
-           >>> assert exc_info.type is ValueError
+           >>> assert exc_info.type == ValueError
 
-    **Using with** ``pytest.mark.parametrize``
 
-    When using :ref:`pytest.mark.parametrize ref`
-    it is possible to parametrize tests such that
-    some runs raise an exception and others do not.
+    Since version ``3.1`` you can use the keyword argument ``match`` to assert that the
+    exception matches a text or regex::
 
-    See :ref:`parametrizing_conditional_raising` for an example.
+        >>> with raises(ValueError, match='must be 0 or None'):
+        ...     raise ValueError("value must be 0 or None")
 
-    **Legacy form**
+        >>> with raises(ValueError, match=r'must be \d+$'):
+        ...     raise ValueError("value must be 42")
+
+    **Legacy forms**
+
+    The forms below are fully supported but are discouraged for new code because the
+    context manager form is regarded as more readable and less error-prone.
 
     It is possible to specify a callable by passing a to-be-called lambda::
 
@@ -652,8 +523,16 @@ def raises(expected_exception, *args, **kwargs):
         >>> raises(ZeroDivisionError, f, x=0)
         <ExceptionInfo ...>
 
-    The form above is fully supported but discouraged for new code because the
-    context manager form is regarded as more readable and less error-prone.
+    It is also possible to pass a string to be evaluated at runtime::
+
+        >>> raises(ZeroDivisionError, "f(0)")
+        <ExceptionInfo ...>
+
+    The string will be evaluated using the same ``locals()`` and ``globals()``
+    at the moment of the ``raises`` call.
+
+    .. autoclass:: _pytest._code.ExceptionInfo
+        :members:
 
     .. note::
         Similar to caught exception objects in Python, explicitly clearing
@@ -671,48 +550,45 @@ def raises(expected_exception, *args, **kwargs):
 
     """
     __tracebackhide__ = True
-    for exc in filterfalse(isclass, always_iterable(expected_exception, BASE_TYPE)):
-        msg = (
-            "exceptions must be old-style classes or"
-            " derived from BaseException, not %s"
-        )
-        raise TypeError(msg % type(exc))
+    msg = ("exceptions must be old-style classes or"
+           " derived from BaseException, not %s")
+    if isinstance(expected_exception, tuple):
+        for exc in expected_exception:
+            if not isclass(exc):
+                raise TypeError(msg % type(exc))
+    elif not isclass(expected_exception):
+        raise TypeError(msg % type(expected_exception))
 
-    message = "DID NOT RAISE {}".format(expected_exception)
+    message = "DID NOT RAISE {0}".format(expected_exception)
     match_expr = None
 
     if not args:
         if "message" in kwargs:
             message = kwargs.pop("message")
-            warnings.warn(deprecated.RAISES_MESSAGE_PARAMETER, stacklevel=2)
         if "match" in kwargs:
             match_expr = kwargs.pop("match")
-        if kwargs:
-            msg = "Unexpected keyword arguments passed to pytest.raises: "
-            msg += ", ".join(sorted(kwargs))
-            raise TypeError(msg)
+            message += " matching '{0}'".format(match_expr)
         return RaisesContext(expected_exception, message, match_expr)
     elif isinstance(args[0], str):
-        warnings.warn(deprecated.RAISES_EXEC, stacklevel=2)
-        (code,) = args
+        code, = args
         assert isinstance(code, str)
         frame = sys._getframe(1)
         loc = frame.f_locals.copy()
         loc.update(kwargs)
         # print "raises frame scope: %r" % frame.f_locals
         try:
-            code = _pytest._code.Source(code).compile(_genframe=frame)
-            exec(code, frame.f_globals, loc)
-            # XXX didn't mean f_globals == f_locals something special?
+            code = _pytest._code.Source(code).compile()
+            py.builtin.exec_(code, frame.f_globals, loc)
+            # XXX didn'T mean f_globals == f_locals something special?
             #     this is destroyed here ...
         except expected_exception:
-            return _pytest._code.ExceptionInfo.from_current()
+            return _pytest._code.ExceptionInfo()
     else:
         func = args[0]
         try:
             func(*args[1:], **kwargs)
         except expected_exception:
-            return _pytest._code.ExceptionInfo.from_current()
+            return _pytest._code.ExceptionInfo()
     fail(message)
 
 
@@ -727,17 +603,24 @@ class RaisesContext(object):
         self.excinfo = None
 
     def __enter__(self):
-        self.excinfo = _pytest._code.ExceptionInfo.for_later()
+        self.excinfo = object.__new__(_pytest._code.ExceptionInfo)
         return self.excinfo
 
     def __exit__(self, *tp):
         __tracebackhide__ = True
         if tp[0] is None:
             fail(self.message)
+        if sys.version_info < (2, 7):
+            # py26: on __exit__() exc_value often does not contain the
+            # exception value.
+            # http://bugs.python.org/issue7853
+            if not isinstance(tp[1], BaseException):
+                exc_type, value, traceback = tp
+                tp = exc_type, exc_type(value), traceback
         self.excinfo.__init__(tp)
         suppress_exception = issubclass(self.excinfo.type, self.expected_exception)
         if sys.version_info[0] == 2 and suppress_exception:
             sys.exc_clear()
-        if self.match_expr is not None and suppress_exception:
+        if self.match_expr:
             self.excinfo.match(self.match_expr)
         return suppress_exception
