@@ -1,53 +1,66 @@
-# -*- coding: utf-8 -*-
 """ log machine-parseable test session result information in a plain
 text file.
 """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-import os
+from __future__ import absolute_import, division, print_function
 
 import py
+import os
 
 
 def pytest_addoption(parser):
     group = parser.getgroup("terminal reporting", "resultlog plugin options")
-    group.addoption(
-        "--resultlog",
-        "--result-log",
-        action="store",
-        metavar="path",
-        default=None,
-        help="DEPRECATED path for machine-readable result log.",
-    )
+    group.addoption('--resultlog', '--result-log', action="store",
+                    metavar="path", default=None,
+                    help="DEPRECATED path for machine-readable result log.")
 
 
 def pytest_configure(config):
     resultlog = config.option.resultlog
     # prevent opening resultlog on slave nodes (xdist)
-    if resultlog and not hasattr(config, "slaveinput"):
+    if resultlog and not hasattr(config, 'slaveinput'):
         dirname = os.path.dirname(os.path.abspath(resultlog))
         if not os.path.isdir(dirname):
             os.makedirs(dirname)
-        logfile = open(resultlog, "w", 1)  # line buffered
+        logfile = open(resultlog, 'w', 1)  # line buffered
         config._resultlog = ResultLog(config, logfile)
         config.pluginmanager.register(config._resultlog)
 
         from _pytest.deprecated import RESULT_LOG
-        from _pytest.warnings import _issue_warning_captured
-
-        _issue_warning_captured(RESULT_LOG, config.hook, stacklevel=2)
+        config.warn('C1', RESULT_LOG)
 
 
 def pytest_unconfigure(config):
-    resultlog = getattr(config, "_resultlog", None)
+    resultlog = getattr(config, '_resultlog', None)
     if resultlog:
         resultlog.logfile.close()
         del config._resultlog
         config.pluginmanager.unregister(resultlog)
 
 
+def generic_path(item):
+    chain = item.listchain()
+    gpath = [chain[0].name]
+    fspath = chain[0].fspath
+    fspart = False
+    for node in chain[1:]:
+        newfspath = node.fspath
+        if newfspath == fspath:
+            if fspart:
+                gpath.append(':')
+                fspart = False
+            else:
+                gpath.append('.')
+        else:
+            gpath.append('/')
+            fspart = True
+        name = node.name
+        if name[0] in '([':
+            gpath.pop()
+        gpath.append(name)
+        fspath = newfspath
+    return ''.join(gpath)
+
+
 class ResultLog(object):
     def __init__(self, config, logfile):
         self.config = config
@@ -59,7 +72,7 @@ class ResultLog(object):
             print(" %s" % line, file=self.logfile)
 
     def log_outcome(self, report, lettercode, longrepr):
-        testpath = getattr(report, "nodeid", None)
+        testpath = getattr(report, 'nodeid', None)
         if testpath is None:
             testpath = report.fspath
         self.write_log_entry(testpath, lettercode, longrepr)
@@ -67,14 +80,12 @@ class ResultLog(object):
     def pytest_runtest_logreport(self, report):
         if report.when != "call" and report.passed:
             return
-        res = self.config.hook.pytest_report_teststatus(
-            report=report, config=self.config
-        )
+        res = self.config.hook.pytest_report_teststatus(report=report)
         code = res[1]
-        if code == "x":
+        if code == 'x':
             longrepr = str(report.longrepr)
-        elif code == "X":
-            longrepr = ""
+        elif code == 'X':
+            longrepr = ''
         elif report.passed:
             longrepr = ""
         elif report.failed:
@@ -95,8 +106,8 @@ class ResultLog(object):
             self.log_outcome(report, code, longrepr)
 
     def pytest_internalerror(self, excrepr):
-        reprcrash = getattr(excrepr, "reprcrash", None)
+        reprcrash = getattr(excrepr, 'reprcrash', None)
         path = getattr(reprcrash, "path", None)
         if path is None:
             path = "cwd:%s" % py.path.local()
-        self.write_log_entry(path, "!", str(excrepr))
+        self.write_log_entry(path, '!', str(excrepr))
